/*
 * $RCSfile: msg_internal.h,v $
 *
 * x-kernel v3.2
 *
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 *
 *
 * $Log: msg_internal.h,v $
 * Revision 1.26.1.1  1994/06/08  23:19:34  davidm
 * MNode*Size definitions are less ad-hoc, derive more from the
 * MNode structure
 *
 */

/* this should be a power of two; see PAGEROUND in msg.c */

#define MSG_STATISTICS

#ifndef XKMACHKERNEL
#define PageSize 4096
#define PAGEROUND(len)  ((len + MNodeLeafSize + (PageSize-1)) / PageSize)
#else
/* this hack is more efficient for funky inkernel malloc/free */
#define PageSize 2048
/* was 4096 -- rcs */
#define PAGEROUND(len)  ((len + MNodeLeafSize + (PageSize-1) + sizeof(void*)) / PageSize)
#endif /* ! XKMACHKERNEL */
/* 
 * Size of virtual space for tree nodes
 */
#ifdef MSGCACHEING
static int MNodePairVSpace = 1 * PageSize;
static int MNodeBufVSpace =  1 * PageSize;
static int MNodePageVSpace = 126 * PageSize;

static char *mWindow;
#endif /* MSGCACHEING */


/* enum NodeType */

enum NodeType {
  t_MNodeJunk,
  t_MNodePair,
  t_MNodePage,
  t_MNodeUsrPage,
  t_MNodeBuf,
  t_MNodeDummy
};

#define mprtype t_MNodePair
#define mpgtype t_MNodePage
#define muptype t_MNodeUsrPage
#define mbftype t_MNodeBuf

/*
 *     the left-right node pair constituent
 */

struct nlink {
  long offset;
  long length;
  MNode  *node;
};

/*
 * struct MNode
 * all tree nodes
 */
struct MNode {
  enum NodeType type;
  unsigned long refCnt;
  union {
    struct {
      struct nlink l, r;
    } pair;
    struct {
      long  size;     /* size of buffer */
      char *data;     /* the buffer */
    } leaf;
    struct {
      long  size;     /* size of buffer */
      char *data;     /* the buffer */
      char buffer[1];	/* actually buffer[MNodePageDataSize] but would be recursive... */
    } page;
    struct {
      long  size;     /* size of buffer */
      char *data;     /* the buffer */
      void (*bFree)(
#ifdef __STDC__
		    void*, int
#endif
		    );  /* the buffer's deallocator function */
    } usrpage;
    struct {
      long  size;     /* size of buffer */
      char *data;     /* the buffer */
      void (*bFree)(
#ifdef __STDC__
		    void*
#endif
		    );  /* the buffer's deallocator function */
    } buf;
  } b;
};

/* These sizes are computed from the MNode structure definition */
#define MNodeSize		((char*)&((struct MNode*)0)->b - (char*)0)
#define MNodePairSize		(MNodeSize + sizeof(((struct MNode*)0)->b.pair))
#define MNodeLeafSize		(MNodeSize + sizeof(((struct MNode*)0)->b.leaf))
#define MNodePageSize		(PageSize)
#define MNodePageDataSize	(MNodePageSize - MNodeLeafSize)
#define MNodeUsrPageSize	(MNodeLeafSize + sizeof(((struct MNode*)0)->b.usrpage))
#define MNodeBufSize		(MNodeLeafSize + sizeof(((struct MNode*)0)->b.buf))

#define mprsize MNodePairSize
#define mpgsize MNodePageSize
#define mupsize MNodeUsrPageSize
#define mbfsize MNodeBufSize

/* non-recursive routines for traversing a message */

/* #define MSG_NON_REC */
#ifdef MSG_NON_REC

#define MSG_NR_STK_SZ (8*1024 - 4)

struct mstack {
  int size;
  MNodePage **top, **bottom, **limit;
  XCharFun func;
  void *arg;
  int indent;
};

#endif /* MSG_NON_REC */
