/*
 * @COPYRIGHT@
 * 
 * x-kernel v3.2
 * 
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 * 
 * @COPYRIGHT@
 *
 * $RCSfile: flamingo.h,v $
 *
 * HISTORY
 * $Log: flamingo.h,v $
 * Revision 1.6  1994/06/16  15:35:11  davidm
 * Changed some of the TURBOchannel related definitions.
 *
 * Revision 1.5  1994/06/11  00:02:53  menze
 * Added copyright notice
 *
 * Revision 1.4  1994/05/25  01:00:55  druschel
 * Added support for TurboChannel option interrupts.
 *
 * Revision 1.3  1994/05/15  00:02:38  davidm
 * TURBOchannel definitions added.
 *
 * Revision 1.2  1994/04/20  00:48:57  davidm
 * Clean up and lots of new definitions.
 *
 * Revision 1.1  1994/04/19  03:02:45  davidm
 * Initial revision
 *
 */
/*
 * The following is based on information published in
 * DEC 3000 300/400/500/600/800 Models, System Programmer's Manual,
 * DEC Part #: EK-D3SYS-PM.A01.
 */
#ifndef flamingo_h
#define flamingo_h

/*
 * Convert between dense and sparse TURBOchannel addresses:
 * To convert from sparse to dense, we need to set bit 28 and
 * double the value in bits 0..27 ('cuz it's a sparse address space...).
 * Vice versa for dense to sparse conversion.
 */
#define SPARSE_MASK	(1<<28)
#define DENSE(sa) \
  ((typeof(sa))(((u_long)(sa) & ~(2*SPARSE_MASK-1)) | \
		((u_long)(sa) & (SPARSE_MASK-1))/2))
#define SPARSE(da) \
  ((typeof(da))(((u_long)(da) & ~(SPARSE_MASK-1)) | SPARSE_MASK | \
		2*((u_long)(da) & (SPARSE_MASK-1))))
/*
 * TURBOchannel base address:
 */
#define	TC_BASE_ADDR	0x100000000	/* dense slot 0 address */
#define TC_SLOT_SIZE	0x020000000

/*
 * TC option slot addresses (dense)
 */

#define TC_OPT0_ADDR 0x100000000
#define TC_OPT1_ADDR 0x120000000
#define TC_OPT2_ADDR 0x140000000
#define TC_OPT3_ADDR 0x160000000
#define TC_OPT4_ADDR 0x180000000
#define TC_OPT5_ADDR 0x1a0000000

/*
 * IOCTL Address Map:
 */
#define IOCTL_SCSI_ADDR		0x1c0000000	/* TCDS chip */
#define IOCTL_SYSROM_ADDR	0x1e0000000
#define IOCTL_REGS_ADDR		0x1e0040000
#define IOCTL_ESAR_ADDR		0x1e0080000	/* Ethernet station addr. */
#define IOCTL_LANCE_ADDR	0x1e00c0000
#define IOCTL_SCC0_ADDR		0x1e0100000
#define IOCTL_SCC1_ADDR		0x1e0180000
#define IOCTL_RTC_ADDR		0x1e0200000
#define IOCTL_ISDN_ADDR		0x1e0280000

/*
 * TURBOchannel and Memory Configuration Control & Status Registers:
 */
#define TCIOSLOT		0x1c2000000	/* I/O slot configuration */
#define TCCONFIG		0x1c2000008	/* TURBOchannel config. */
#define TCFADR			0x1c2000010	/* TC failing address */
#define TCEREG			0x1c2000018	/* TC error register */
#define MEM_CONF		0x1c2200000	/* memory config. reg. */
#define TCIMR_READ		0x1c2400000	/* TC intr. mask reg. */
#define TCSGM			0x1c2800000	/* TC scatter/gather map */
#define TCIMR_WRITE		0x1c281fffc	/* TC intr. mask reg. */
#define TCIR			0x1d4800000	/* TC interrupt reg. */
#define TCIR_CLEAR		0x1d4c00000	/* read TC intr. reg & clear */

/*
 * IOCTL Registers Address Map:
 */
#define LANCE_DMA_PTR		0x1e0040020
#define COMM_TX_DMA_PTR		0x1e0040030	/* scc-0 ch B tx DMA ptr */
#define COMM_RX_DMA_PTR		0x1e0040040	/* scc-0 ch B rx DMA ptr */
#define PRNT_TX_DMA_PTR		0x1e0040050	/* scc-1 ch B tx DMA ptr */
#define PRNT_RX_DMA_PTR		0x1e0040060	/* scc-1 ch B rx DMA ptr */
#define ISDN_TX_DMA_PTR		0x1e0040080
#define ISDN_TX_BUF_DMA_PTR	0x1e0040090
#define ISDN_RX_DMA_PTR		0x1e00400a0
#define ISDN_RX_BUF_DMA_PTR	0x1e00400b0
#define SSR			0x1e0040100	/* system support register */
#define SIR			0x1e0040110	/* system intr register */
#define SIMR			0x1e0040120	/* system intr mask register */
#define SAR			0x1e0040130	/* system addr register */
#define ISDN_TX_DATA		0x1e0040140	/* ISDN data tx register */
#define ISDN_RX_DATA		0x1e0040140	/* ISDN data rx register */
#define LANCE_IO_SLOT		0x1e0040160	/* system LANCE I/O slot */
#define SCC0_DMA_SLOT		0x1e0040180	/* system scc-0 DMA slot */
#define SCC1_DMA_SLOT		0x1e0040190	/* system scc-1 DMA slot */

/*
 * System support register bit definitions and masks:
 */
#define SSR_DIAG_LED_MASK	0x000000ff	/* diagnostic LEDs */
#define SSR_LANCE_RESET_OFF	0x00000100	/* LANCE reset off */
#define SSR_RTC_RESET_OFF	0x00000400	/* real time clock reset off */
#define SSR_SCC_RESET_OFF	0x00000800	/* scc0/scc1 reset off */
#define SSR_ISDN_RESET_OFF	0x00001000	/* ISDN/audio reset off */
#define SSR_LANCE_10BASET	0x00002000	/* 10Base-T vs. Thickwire */
#define SSR_LANCE_LOOPBACK_OFF	0x00004000	/* disable loopback mode */
#define SSR_LANCE_TPIC		0x00008000	/* for testing purposes */
#define SSR_LANCE_DMA_ENABLE	0x00010000	/* enable LANCE DMA */
#define SSR_ISDN_RX_DMA_ENABLE	0x00080000	/* enable ISDN receive DMA */
#define SSR_ISDN_TX_DMA_ENABLE	0x00100000	/* enable ISDN transmit DMA */
#define SSR_ROM_BYTE_SELECT	0x03000000	/* single access byte select */
#define SSR_ROM_SINGLE_ACCESS	0x04000000	/* enable single access mode */
#define SSR_SLOW_MODE		0x08000000	/* select slow 12.5 MHz mode */
#define SSR_PRNT_RX_DMA_ENABLE	0x10000000	/* scc-1 ch B rx DMA enable */
#define SSR_PRNT_TX_DMA_ENABLE	0x20000000	/* scc-1 ch B tx DMA enable */
#define SSR_COMM_RX_DMA_ENABLE	0x40000000	/* scc-0 ch B rx DMA enable */
#define SSR_COMM_TX_DMA_ENABLE	0x80000000	/* scc-0 ch B tx DMA enable */

/*
 * System Interrupt Register:
 *
 * Access-Codes: R=read only, W0C=write zero to clear.
 */
#define SIR_CONSOLE_ALTERNATE	0x00000008	/* R alt. console selected */
#define SIR_INTR_SCC0		0x00000040	/* R scc0 intr request */
#define SIR_INTR_SCC1		0x00000080	/* R scc1 intr request */
#define SIR_INTR_LANCE		0x00000100	/* R LANCE intr request */
#define SIR_CONSOLE_SECURE	0x00000200	/* R secure console mode */
#define SIR_INTR_ISDN		0x00002000	/* R ISDN intr request */
#define SIR_LANCE_DMA_MEMERR	0x00010000	/* R/W0C DMA mem read error */
#define SIR_LANCE_DMA_OVERRUN	0x00100000	/* R/W0C DMA overrun */
#define SIR_ISDN_RX_DMA_LD	0x00200000	/* R/W0C rx DMA ptr loaded */
#define SIR_ISDN_TX_DMA_LD	0x00400000	/* R/W0C tx DMA ptr loaded */
#define SIR_PRNT_RX_DMA_OVERRUN 0x01000000	/* R/W0C */
#define SIR_PRNT_RX_DMA_HALF_PG	0x02000000	/* R/W0C */
#define SIR_PRNT_TX_DMA_MEMERR	0x04000000	/* R/W0C */
#define SIR_PRNT_TX_DMA_END_PG	0x08000000	/* R/W0C */
#define SIR_COMM_RX_DMA_OVERRUN	0x10000000	/* R/W0C */
#define SIR_COMM_RX_DMA_HALF_PG	0x20000000	/* R/W0C */
#define SIR_COMM_TX_DMA_MEMERR	0x40000000	/* R/W0C */
#define SIR_COMM_TX_DMA_END_PG	0x80000000	/* R/W0C */

#endif /* flamingo_h */
